%%  Demonstrating the Runge Phenomenon
%   This script demonstrates the Runge phenomenon when using Lagrange
%   interpolation for varying N with equidistant nodes. It also shows how
%   the issue can be resolved when using Chebyshev points

%   Some elements of this script are adapted from Lloyd's N. Trefethen's
%   Book "Spectral Methods in Matlab", Chapter 5: Polynomial Interpolation
%   and Clustered Grids

%   Maximilian Jentzsch
%   Math 56 Final Project

%% Runge Phenomenon

close all
format long g

% number of interpolation points
N = [10 20 50 60];
u = -1.01:0.003:1.01;
max_err=[];
for n = 1:length(N) 
    
    h = 2/(N(n)-1);
    x = -1:h:1;
    y = 1./(1+12*x.^2);
    v = barylagrangeinterp(x,y,u);
    max_err = [max_err norm(v-u,inf)];
    
  figure(1)
    subplot(2,2,n)
        plot(x,y,'.','MarkerSize',12)
        hold on
        plot(u,v,'r-','LineWidth',1.5)
        axis([-1.1 1.1 -1 1.5])
        N_num = N(n);
        title(['N=',num2str(N_num)],'FontSize',16,'FontWeight','bold')
        xlabel('x','FontWeight','bold')
        ylabel('y','FontWeight','bold')
        set(gca,'FontSize',12,'FontWeight','bold')
      
       
end

figure(2)
plot(N,max_err,'LineWidth',3)
title('Max. error of interpolation, equispaced points','FontSize',14,'FontWeight','bold')
xlabel('N','FontSize',12,'FontWeight','bold')
ylabel('Maximum Error','FontSize',12,'FontWeight','bold')
set(gca,'box','off')
set(gca,'FontSize',12,'FontWeight','bold')

%% Fix using Chebyshev points

N = [10 20 50 60];
u = -1.01:0.003:1.01;
max_err1 =[];

for n = 1:length(N) 
    
    % instead of using equidistant nodes, use Chebyshev nodes
    x = cos(pi*(0:N(n))/N(n));
    
    y = 1./(1+12*x.^2);
    v = barylagrangeinterp(x,y,u);
    max_err1 = [max_err1 norm(v-u,inf)];
   
        figure(3)
        subplot(2,2,n)
        plot(x,y,'.','MarkerSize',12)
        hold on
        plot(u,v,'r-','LineWidth',1.5)
        axis([-1.1 1.1 -1 1.5])
        N_num = N(n);
        title(['N=',num2str(N_num)],'FontSize',16,'FontWeight','bold')
        xlabel('x','FontWeight','bold')
        ylabel('y','FontWeight','bold')
        set(gca,'FontSize',12,'FontWeight','bold')
      
       
end







