function [y, ff] = barylagtref(f,n)
%% baryleg - barycentric lagrange interpolation 

%  This function does the barycentric lagrange interpolation with Chebyshev
%  points, taken from Barycentric Lagrange Interpolation by Berrut and
%  Trefethen (2004). It makes use of the easy calculation of the weights
%  w_j.

% Inputs: f is a function handle, and n is the number of interpolation
%         points wanted.
% Outputs: vector y is is the function evaluated as the Chebyshev points,
%          ff is the interpolation vector.

% Optional: Uncommenting the figure section will give an interpolation
%           plot.


% Maximilian Jentzsch
% Math 56 Final Project, Spring 2014.

%% Interpolation

% make a grid of Chebyshev points - these will be the nodes for
% interpolation
  x = cos(pi*(0:n)'/n);
% get y values  
  y = f(x);
% Using Chebyshev nodes makes computing the weights very easy!
  w = [1/2; ones(n-1,1); 1/2].*(-1).^((0:n)');
% Need n+1 so that the vectors will have the same length at the end
  xx = linspace(-1,1,n+1)';
  numer = zeros(size(xx));
  denom = zeros(size(xx));
% when xdiff below becomes zero (often happens at the ends), the
% interpolation value will be inf because the algorithm will divide by
% zero. Exact will store where interpolation and data points are equal. 
  exact = zeros(size(xx));
  for j = 1:n
      xdiff = xx-x(j);
      temp = w(j)./xdiff;
      numer = numer + temp*y(j);
      denom = denom + temp;
      exact(xdiff==0) = 1;
  end
    
    ff = numer./denom;
    jj=find(exact);
    ff(jj)=y(exact(jj));
    y = f(x);

%%   Figure
%     figure
%     plot(x,y,'.',xx,ff)
end

